/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): offroy ________________________.
Contributor(s): ______________________________________.

====================================================================*/
/*
 * Created on 23 mai 2003 by jerome OFFROY (offroy@lifl.fr)
 *
 */

package org.omg.lifl.eclipse.plugin.project.OpenCCM;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.actions.WorkspaceModifyDelegatingOperation;
import org.eclipse.ui.dialogs.IOverwriteQuery;
import org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard;
import org.eclipse.ui.wizards.newresource.BasicNewResourceWizard;
import org.omg.lifl.eclipse.plugin.project.OpenCCM.utils.ProjectMessages;

/**
 * @author LIFL
 *
 */
public class ProjectCreationWizard
    extends BasicNewResourceWizard
    implements INewWizard, IExecutableExtension {

    private IConfigurationElement fConfigElement;

    private ProjectCreationWizardPage[] fPages;

    public ProjectCreationWizard() {
        super();
        setDialogSettings(MainPlugin.getDefault().getDialogSettings());
        setWindowTitle(
            ProjectMessages.getString("ProjectCreationWizard.title"));
        setNeedsProgressMonitor(true);
    }

    /*
     * @see BasicNewResourceWizard#initializeDefaultPageImageDescriptor
     */
    protected void initializeDefaultPageImageDescriptor() {
        if (fConfigElement != null) {
            String banner = fConfigElement.getAttribute("banner");
            if (banner != null) {
                ImageDescriptor desc =
                    MainPlugin.getDefault().getImageDescriptor(banner);
                setDefaultPageImageDescriptor(desc);
            }
        }
    }

    /*
     * @see Wizard#addPages
     */
    public void addPages() {
        super.addPages();

        IConfigurationElement[] children =
            fConfigElement.getChildren("projectsetup");
        if (children == null || children.length == 0) {
            MainPlugin.log(
                "descriptor must contain one ore more projectsetup tags");
            return;
        }

        fPages = new ProjectCreationWizardPage[children.length];

        for (int i = 0; i < children.length; i++) {
            IConfigurationElement curr = children[i];
            fPages[i] = new ProjectCreationWizardPage(i, children[i]);
            addPage(fPages[i]);
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.wizard.IWizard#performFinish()
     */
    public boolean performFinish() {
        ProjectCreationOperation runnable =
            new ProjectCreationOperation(fPages, new ImportOverwriteQuery());

        IRunnableWithProgress op =
            new WorkspaceModifyDelegatingOperation(runnable);
        try {
            getContainer().run(false, true, op);
        } catch (InvocationTargetException e) {
            handleException(e.getTargetException());
            return false;
        } catch (InterruptedException e) {
            return false;
        }
        BasicNewProjectResourceWizard.updatePerspective(fConfigElement);
        IResource res = runnable.getElementToOpen();
        if (res != null) {
            openResource(res);
        }
        return true;
    }

    /**
     * @param res
     */
    private void openResource(final IResource resource) {
        if (resource.getType() != IResource.FILE) {
            return;
        }
        IWorkbenchWindow window =
            MainPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow();
        if (window == null) {
            return;
        }
        final IWorkbenchPage activePage = window.getActivePage();
        if (activePage != null) {
            final Display display = getShell().getDisplay();
            display.asyncExec(new Runnable() {
                public void run() {
                    try {
                        activePage.openEditor((IFile) resource);
                    } catch (PartInitException e) {
                        MainPlugin.log(e);
                    }
                }
            });
            selectAndReveal(resource);
        }
    }

    /**
     * @param throwable
     */
    private void handleException(Throwable target) {
        String title =
            ProjectMessages.getString("ProjectCreationWizard.op_error.title");
        String message =
            ProjectMessages.getString("ProjectCreationWizard.op_error.message");
        if (target instanceof CoreException) {
            IStatus status = ((CoreException) target).getStatus();
            ErrorDialog.openError(getShell(), title, message, status);
            MainPlugin.log(status);
        } else {
            MessageDialog.openError(getShell(), title, target.getMessage());
            MainPlugin.log(target);
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.core.runtime.IExecutableExtension#setInitializationData(org.eclipse.core.runtime.IConfigurationElement, java.lang.String, java.lang.Object)
     */
    public void setInitializationData(
        IConfigurationElement cfig,
        String propertyName,
        Object data) {
        fConfigElement = cfig;

    }

    // overwrite dialog

    private class ImportOverwriteQuery implements IOverwriteQuery {
        public String queryOverwrite(String file) {
            String[] returnCodes = { YES, NO, ALL, CANCEL };
            int returnVal = openDialog(file);
            return returnVal < 0 ? CANCEL : returnCodes[returnVal];
        }

        private int openDialog(final String file) {
            final int[] result = { IDialogConstants.CANCEL_ID };
            getShell().getDisplay().syncExec(new Runnable() {
                public void run() {
                    String title =
                        ProjectMessages.getString(
                            "ProjectCreationWizard.overwritequery.title");
                    String msg =
                        ProjectMessages.getFormattedString(
                            "ProjectCreationWizard.overwritequery.message",
                            file);
                    String[] options =
                        {
                            IDialogConstants.YES_LABEL,
                            IDialogConstants.NO_LABEL,
                            IDialogConstants.YES_TO_ALL_LABEL,
                            IDialogConstants.CANCEL_LABEL };
                    MessageDialog dialog =
                        new MessageDialog(
                            getShell(),
                            title,
                            null,
                            msg,
                            MessageDialog.QUESTION,
                            options,
                            0);
                    result[0] = dialog.open();
                }
            });
            return result[0];
        }
    }

}
