/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.corba.generator.psdl.lib;

/** To access AST. */
import org.objectweb.corba.ast.api.StorageTypeDecl;
import org.objectweb.corba.ast.api.AbstractStorageTypeDecl;
import org.objectweb.corba.ast.api.StorageTypeStateMemberDecl;
import org.objectweb.corba.ast.api.PsdlOperationDecl;
import org.objectweb.corba.ast.api.DeclarationKind;
import org.objectweb.corba.ast.api.Declaration;

/** To access Java AST. */
import org.objectweb.corba.generator.java.ast.api.*;
import org.objectweb.corba.generator.java.ast.lib.*;

/** Others. */
import java.util.Vector;
import org.objectweb.corba.generator.common.lib.GenerationException;


/**
 * This class generates Storage Type Mapping.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 */

public class StorageTypeMapping
  implements org.objectweb.corba.generator.psdl.api.StorageTypeMapping
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The storage type declaration. */
    private StorageTypeDecl st_;

    /** All state members to map. */
    private java.util.List all_states_;

    /** Read only state members. */
    private java.util.List read_states_;

    // ===========================================================
    //
    // Constructors.
    //
    // ===========================================================

    /**
     * The default constructor.
     *
     * @param ast - The Abstract Syntax Tree.
     */

    public StorageTypeMapping(StorageTypeDecl st)
    {
        // Init internal state
        st_ = st;
        read_states_ = new java.util.ArrayList();
        all_states_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Has this abstract storage type an operation declaration ?
     * (including implemented abstract storage types)
     *
     * @param ast - The abstract storage type to test.
     *
     * @return True if an operation is contained, else false.
     **/
    private boolean
    hasOperation(AbstractStorageTypeDecl ast)
    {
        Declaration[] decls = null;
        decls = ast.getContents(true, DeclarationKind.dk_psdl_operation);
        if (decls.length > 0)
        {
            return true;
        }
        else
        {
            // Check implemented abstract storage types
            AbstractStorageTypeDecl[] implemented = null;

            implemented = ast.getAbstractStorageTypeList().getAbstractStorageTypes();
            for (int i=0; i<implemented.length; i++)
            {
                if ( hasOperation(implemented[i]) )
                    return true;
            }
            return false;
        }
    }

    /**
     * Add methods of javax.jdo.InstanceCallbacks interface to this storage type.
     *
     * @param generator - The PSDL to Java generator.
     * @param clazz - The abstract storage type mapping class.
     **/
    private void
    addJdoInstanceCallbacks(org.objectweb.corba.generator.psdl.api.PSDL2JavaGenerator generator,
                            ClassObject clazz)
    {
        java.util.List to_codec = null,
                       to_ior = null,
                       to_class = null;
        java.util.Iterator it = null;
        StorageTypeStateMemberDecl state = null;
        String mapping_type = null;
        MethodObject method = null;
        org.objectweb.corba.generator.java.api.JavaTranslator translator = null;

        translator = generator.getTranslator();

        clazz.addImplementedObject("javax.jdo.InstanceCallbacks");

        // Compute marshalled states
        to_codec = getStatesToCodec(generator);
        to_ior = getStatesToIOR(generator);
        to_class = getStatesToClass(generator);

        // Add the jdoPostLoad method
        method = new MethodObjectImpl();
        method.addComment("javax.jdo.InstanceCallbacks jdoPostLoad method.");
        method.setName("jdoPostLoad");
        method.setReturnType("void");
        method.getImpl().setMacro("ST_JDO_POST_LOAD_METHOD");
        method.getImpl().addContextValue("to_codec", to_codec);
        method.getImpl().addContextValue("to_ior", to_ior);
        method.getImpl().addContextValue("to_class", to_class);
        method.getImpl().addContextValue("translator", translator);
        clazz.addMethod(method);

        // Add the jdoPreStore method
        method = new MethodObjectImpl();
        method.addComment("javax.jdo.InstanceCallbacks jdoPreStore method.");
        method.setName("jdoPreStore");
        method.setReturnType("void");
        method.getImpl().setMacro("ST_JDO_PRE_STORE_METHOD");
        method.getImpl().addContextValue("to_codec", to_codec);
        method.getImpl().addContextValue("to_ior", to_ior);
        method.getImpl().addContextValue("to_class", to_class);
        method.getImpl().addContextValue("translator", translator);
        clazz.addMethod(method);

        // Add the jdoPreClear method
        method = new MethodObjectImpl();
        method.addComment("javax.jdo.InstanceCallbacks jdoPreClear method.");
        method.setName("jdoPreClear");
        method.setReturnType("void");
        clazz.addMethod(method);

        // Add the jdoPreDelete method
        method = new MethodObjectImpl();
        method.addComment("javax.jdo.InstanceCallbacks jdoPreDelete method.");
        method.setName("jdoPreDelete");
        method.getImpl().setMacro("ST_JDO_PRE_DELETE_METHOD");
        method.getImpl().addContextValue("to_class", to_class);
        method.getImpl().addContextValue("translator", translator);
        method.setReturnType("void");
        clazz.addMethod(method);
    }

    /**
     * Map state members.
     *
     * @param state_members - State Mebers to map.
     * @param clazz - The abstract storage type mapping class.
     * @param ct - The abstract storage type constructor mapping.
     * @param generator - The PSDL to Java generator.
     **/
    private void
    mapStateMembers(StorageTypeStateMemberDecl[] state_members,
                    ClassObject clazz,
                    ConstructorObject ct,
                    org.objectweb.corba.generator.psdl.api.PSDL2JavaGenerator generator)
    {
        for (int i=0; i<state_members.length; i++)
        {
            org.objectweb.corba.generator.psdl.api.StateMemberMapping state
                = new StateMemberMapping(state_members[i]);

            state.toJavaAccessors(generator.getTranslator(), clazz, true);
            state.toJavaAttribute(generator.getTranslator(), clazz, st_);
            if ( state_members[i].isReadonly() )
            {
                state.toJavaParameter(generator.getTranslator(), ct);
                read_states_.add(state_members[i].getName());
                state.toJavaReadOnlyStateMemberModifier(generator.getTranslator(), clazz);
            }
       }
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Get the list of all state members that will be mapped.
     *
     * @param generator - A base generator to get declarationss.
     *
     * @return The list of all state members.
     **/
    public java.util.List
    getAllStates(org.objectweb.corba.generator.common.api.GeneratorBase generator)
    {
        AbstractStorageTypeDecl[] impl_directly = null;;
        Vector vect = null;

        if (all_states_ == null)
        {
            all_states_ = new java.util.ArrayList();

            // Add State Members implemented directly
            impl_directly = st_.getDirectlyImplementedAbstractStorageTypes();
            for (int j=0; j<impl_directly.length; j++)
            {
                try{
                    vect = generator.getDeclarations(impl_directly[j], DeclarationKind.dk_storage_type_state_member);
                }catch(org.objectweb.corba.generator.common.lib.GenerationException ex){
                    ex.printStackTrace();
                }
                all_states_.addAll( vect );
            }

            // Add own State Members
            try{
                vect = generator.getDeclarations(st_, DeclarationKind.dk_storage_type_state_member);
            }catch(org.objectweb.corba.generator.common.lib.GenerationException ex){
                ex.printStackTrace();
            }
            all_states_.addAll( vect );
        }
        return all_states_;
    }

    /**
     * Get the list of all state members that will be mapped to native types.
     *
     * @param generator - A base generator to get declarationss.
     *
     * @return The list of all state members mapped to native types.
     **/
    public java.util.List
    getStatesToNative(org.objectweb.corba.generator.common.api.GeneratorBase generator)
    {
        StorageTypeStateMemberDecl state = null;
        String mapping_type = null;
        java.util.List to_native = null;
        java.util.Iterator it = null;

        to_native = new java.util.ArrayList();
        it = getAllStates(generator).iterator();
        while (it.hasNext())
        {
            state = (StorageTypeStateMemberDecl) it.next();
            mapping_type = PSDLMappingTools.getMarshalledType(st_, state);
            if (mapping_type == null)
            {
                to_native.add(state);
            }
        }
        return to_native;
    }

    /**
     * Get the list of all state members that will be mapped to an IOR.
     *
     * @param generator - A base generator to get declarationss.
     *
     * @return The list of all state members mapped to an IOR.
     **/
    public java.util.List
    getStatesToIOR(org.objectweb.corba.generator.common.api.GeneratorBase generator)
    {
        StorageTypeStateMemberDecl state = null;
        String mapping_type = null;
        java.util.List to_ior = null;
        java.util.Iterator it = null;

        to_ior = new java.util.ArrayList();
        it = getAllStates(generator).iterator();
        while (it.hasNext())
        {
            state = (StorageTypeStateMemberDecl) it.next();
            mapping_type = PSDLMappingTools.getMarshalledType(st_, state);
            if ( (mapping_type != null) &&
                 (mapping_type.compareTo("java.lang.String") == 0) )
            {
                to_ior.add(state);
            }
        }
        return to_ior;
    }

    /**
     * Get the list of all state members that will be mapped to a byte array.
     *
     * @param generator - A base generator to get declarationss.
     *
     * @return The list of all state members mapped to a byte array.
     **/
    public java.util.List
    getStatesToCodec(org.objectweb.corba.generator.common.api.GeneratorBase generator)
    {
        StorageTypeStateMemberDecl state = null;
        String mapping_type = null;
        java.util.List to_codec = null;
        java.util.Iterator it = null;

        to_codec = new java.util.ArrayList();
        it = getAllStates(generator).iterator();
        while (it.hasNext())
        {
            state = (StorageTypeStateMemberDecl) it.next();
            mapping_type = PSDLMappingTools.getMarshalledType(st_, state);
            if ( (mapping_type != null) &&
                 (mapping_type.compareTo("byte[]") == 0) )
            {
                to_codec.add(state);
            }
        }
        return to_codec;
    }

    /**
     * Get the list of all state members that will be mapped to a byte array.
     *
     * @param generator - A base generator to get declarationss.
     *
     * @return The list of all state members mapped to a byte array.
     **/
    public java.util.List
    getStatesToClass(org.objectweb.corba.generator.common.api.GeneratorBase generator)
    {
        StorageTypeStateMemberDecl state = null;
        String mapping_type = null;
        java.util.List to_class = null;
        java.util.Iterator it = null;

        to_class = new java.util.ArrayList();
        it = getAllStates(generator).iterator();
        while (it.hasNext())
        {
            state = (StorageTypeStateMemberDecl) it.next();
            mapping_type = PSDLMappingTools.getMarshalledType(st_, state);
            if ( (mapping_type != null) &&
                 (mapping_type.compareTo("java.lang.String") != 0) &&
                 (mapping_type.compareTo("byte[]") != 0) )
            {
                to_class.add(state);
            }
        }
        return to_class;
    }

    /**
     * Generate a java persistent capable class for this storage type.
     *
     * @param storage_object - If true, generates mapping for a storage object.
     * @param generator - The PSDL to Java generator.
     */
    public void
    toJava(boolean storage_object,
           org.objectweb.corba.generator.psdl.api.PSDL2JavaGenerator generator)
    throws GenerationException
    {
        ClassObject clazz = null;
        ConstructorObject ct = null;
        MethodObject method = null;
        AttributeObject att = null;
        ParameterObject param = null;
        AbstractStorageTypeDecl[] implemented = null,
                                  impl_directly = null;;
        StorageTypeDecl base = null;
        Vector vect = null;
        StorageTypeStateMemberDecl[] state_members = null;
        PsdlOperationDecl[] operations = null;

        read_states_ = new java.util.ArrayList();

        clazz = new ClassObjectImpl(st_.getName());
        clazz.addComment(" Storage Type "+st_.getName());
        clazz.addComment("");
        clazz.addComment(" @author OpenCCM PSDL Compiler");
        clazz.setModifier(ModifierKindImpl.mk_public);
        clazz.setPackage(generator.getTranslator().getPackage(st_));

        // Add Implemented Abstract Storage Type by st
        implemented = st_.getAbstractStorageTypeList().getAbstractStorageTypes();
        for (int i=0; i<implemented.length; i++)
        {
            clazz.addImplementedObject(generator.getTranslator().getAbsoluteName(implemented[i]));
        }

        // Check if any of the abstract storagetypes implemented declares an operation
        int i = 0;
        boolean found = false;
        clazz.setAbstract(false);
        while ((i<implemented.length) && !found)
        {
            if (hasOperation(implemented[i]))
            {
                clazz.setAbstract(true);
                found = true;
            }
            i++;
        }

        // Add Inherited Storage Type if there is one
        base = st_.getBaseStorageType();
        if (base != null)
            clazz.addInheritedObject(generator.getTranslator().getAbsoluteName(base));
        else if (storage_object)
            clazz.addInheritedObject("org.objectweb.openccm.services.pss.jdo.lib.StorageObjectBase");

        // Add the default constructor
        ct = new ConstructorObjectImpl();
        ct.getImpl().setMacro("ST_CONSTRUCTOR");
        clazz.addConstructor(ct);

        if (storage_object)
        {
            // Add the _pid_state attribute
            att = new AttributeObjectImpl();
            att.addComment("The pid state.");
            att.setName("_pid_state");
            att.setType("byte[]");
            att.setModifier( ModifierKindImpl.mk_public );
            clazz.addAttribute(att);

            // Add the private set_pid_state method
            method = new MethodObjectImpl();
            method.addComment("Set the pid state.");
            method.setModifier( ModifierKindImpl.mk_private );
            method.setName("set_pid_state");
            method.setReturnType("void");
            param = new ParameterObjectImpl();
            param.setName("pid");
            param.setType("byte[]");
            method.addParameter(param);
            method.getImpl().setMacro("SET_METHOD");
            method.getImpl().addContextValue("var", "_pid_state");
            method.getImpl().addContextValue("value", "pid");
            clazz.addMethod(method);

            // Override the initialize method
            method = new MethodObjectImpl();
            method.addComment("Initialize the storage object.");
            method.setName("initialize");
            method.setReturnType("void");
            param = new ParameterObjectImpl();
            param.setName("storage_home");
            param.setType("org.omg.CosPersistentState.StorageHomeBase");
            method.addParameter(param);
            param = new ParameterObjectImpl();
            param.setName("pid");
            param.setType("byte[]");
            method.addParameter(param);
            method.getImpl().setMacro("ST_INITIALIZE_METHOD");
            clazz.addMethod(method);
        }

        java.util.List all_states = getAllStates(generator);
        state_members = (StorageTypeStateMemberDecl[])all_states.toArray(new StorageTypeStateMemberDecl[0]);
        mapStateMembers(state_members, clazz, ct, generator);

        // Complete the storage type constructor
        ct.getImpl().addContextValue("read_states", read_states_);
        // Add store directives
        vect = generator.getDeclarations(st_, DeclarationKind.dk_storage_type_store_directive);
        ct.getImpl().addContextValue("store_directives", vect);
        ct.getImpl().addContextValue("translator", generator.getTranslator());

        addJdoInstanceCallbacks(generator, clazz);

        generator.getJavaRepository().addObject(clazz);
    }
}
